# from __future__ import annotations
from typing import List, Set

from introspection import ModelRepresentation, ModelFieldRepresentation

from ..utils import field_ts_type_from_classname, to_camel_case_var, to_snake_case


class FrontendModel:
    model: ModelRepresentation

    def __init__(self, model: ModelRepresentation) -> None:
        """Initialize from a model representation

        :param model: a model representation from django-introspection
        :type model: ModelRepresentation
        """
        self.model = model

    @property
    def contract_import(self) -> str:
        """Get the Typescript contract import for this model

        :return: a Typescript import string
        :rtype: str
        """
        return f'import {self.model.name}Contract from "./contract"'

    @property
    def api_relative_import(self) -> str:
        """Get the Typescript api import

        :return: a Typescript import string
        :rtype: str
        """
        return 'import api from "../../api"'

    @property
    def snake_case_name(self) -> str:
        """Convert the model name to snake case

        :return: the model name in snake case
        :rtype: str
        """
        return to_snake_case(self.model.name)

    def interface(self) -> str:
        """Get a Typescript interface for a model

        :return: the Typescript interface for the model
        :rtype: str
        """
        buf: List[str] = []
        extra_imports: Set[str] = set()
        buf.append(f"export default interface {self.model.name}Contract " + "{")
        for field in self.model.fields.values():
            buf.append(self.field_type(field) + ",")
            if field.is_relation is True:
                extra_imports.add(
                    self.relation_contract_import(
                        field.related_class_name,
                        to_snake_case(field.related_class_name),
                    )
                )
        buf.append("}")
        output = ""
        main = "\n".join(buf)
        if len(extra_imports) > 0:
            output = ";\n".join(extra_imports) + ";\n\n"
        output += main
        return output

    def constructor(self) -> str:
        """
        Get a Typescript constructor for a model
        """
        buf: List[str] = []
        params: List[str] = []
        main: List[str] = []
        for field in self.model.fields.values():
            main.append(f"\t\tthis.{field.name} = {field.name}")
            params.append(f"{field.name}")
        c_begin = "\tconstructor ({ "
        c_end = f": {self.model.name}Contract) {'{'}"
        buf.append(c_begin + ", ".join(params) + " }" + c_end)
        buf.append(";\n".join(main))
        buf.append("\t}")
        return "\n".join(buf)

    def tsclass(self) -> str:
        """
        Render a Typescript class for a model
        """
        interface = self.contract_import
        extra_imports: Set[str] = set()
        buf: List[str] = [interface + ";\n"]
        buf.append(f"export default class {self.model.name} {'{'}")
        for field in self.model.fields.values():
            buf.append(self.field_type(field) + ";")
            if field.is_relation is True:
                extra_imports.add(
                    self.relation_contract_import(
                        field.related_class_name,
                        to_snake_case(field.related_class_name),
                    )
                )
        buf.append(f"\n{self.constructor()}")
        buf.append("\n" + self.from_json_method())
        buf.append("}")
        output = ""
        if len(extra_imports) > 0:
            output = ";\n".join(extra_imports) + ";\n"
        output += "\n".join(buf)
        return output

    def from_json_method(self) -> str:
        """
        Get the to_json method for a Typescript class
        """
        buf: List[str] = [
            (
                f"\tstatic fromJson(data: Record<string, any>):"
                f" {self.model.name}"
                " {"
            )
        ]
        buf.append(
            f"\t\treturn new {self.model.name}(data as {self.model.name}Contract)"
        )
        buf.append("\t}")
        return "\n".join(buf)

    def load_method(self) -> str:
        """
        Get the load method for a Typescript class
        """
        buf: List[str] = [
            (
                f"\tstatic async load(id: number | string): Promise<{self.model.name}>"
                " {"
            )
        ]
        line = (
            "\t\tconst res = await api.get<Record<string, any>>(`/api/"
            f"{self.snake_case_name}/$"
            "{id}/`);"
        )
        buf.append(line)
        buf.append(f"\t\treturn {self.model.name}.fromJson(res)")
        buf.append("\t}")
        return "\n".join(buf)

    def field_type(self, field: ModelFieldRepresentation) -> str:
        """
        Get the typescript field type definition string
        """
        f = ""
        if field.is_relation is False:
            f = f"\t{field.name}: {field_ts_type_from_classname(field.classname)}"
        else:
            f = self.field_relation_type(field)
        return f

    def field_relation_type(self, field: ModelFieldRepresentation) -> str:
        """
        Get the typescript field type definition string
        for a relation field
        """
        val = f"{field.related_class_name}Contract"
        if field.classname == "ManyToManyField":
            val = f"Array<{field.related_class_name}Contract>"
        if field.is_null is True:
            val += " | null"
        return f"\t{to_camel_case_var(field.name)}: {val}"

    def relation_contract_import(
        self, relation_class_name: str, relation_model_name: str
    ) -> str:
        """
        Get an import line for a given model contract
        """
        return (
            f'import {relation_class_name}Contract from "../'
            f'{relation_model_name}/contract"'
        )



import json
from typing import Union

from django.contrib.auth import login, authenticate, logout

from django.http.response import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.http import JsonResponse, HttpResponseForbidden, HttpRequest
from django.middleware.csrf import rotate_token
from django.conf import settings


@csrf_exempt  # type: ignore
def login_view(request: HttpRequest) -> HttpResponse:
    """Login user from username/password and get information about
    the authorized orgs for this user. If the user is already logged
    in return this information. If the user has only a session cookie
    and no csrf cookie set a csrf cookie

    :param request: the post request
    :type request: HttpRequest
    :return: the json response
    :rtype: HttpResponse
    """
    print("Login view", request.method)
    if request.method == "POST":
        if request.user.is_authenticated:  # type: ignore
            csrf: Union[str, None] = request.COOKIES.get(
                settings.CSRF_COOKIE_NAME, None
            )
            # print("Csrf:", csrf)
            # reset the csrf token cookie if needed
            if csrf is None:
                print("Renewing csrf token because user does not have any csrf cookie")
                rotate_token(request)
            return JsonResponse({"ok": True})
        # print("POST", request.body)
        json_data = json.loads(request.body)
        print("LOGIN", json_data)
        username = json_data["username"]
        password = json_data["password"]
        print("Authenticate", username, password)
        user = authenticate(username=username, password=password)
        if user is not None:
            login(request, user)
            return JsonResponse({"ok": True})
    return HttpResponseForbidden()


def logout_view(request: HttpRequest) -> HttpResponse:
    """Logout a user

    :param request: the get request
    :type request: HttpRequest
    :return: the json response
    :rtype: HttpResponse
    """
    if request.user.is_authenticated is True:  # type: ignore
        logout(request)
    return JsonResponse({"ok": True})



import json
from dataclasses import asdict, dataclass, field
from pathlib import Path


@dataclass(frozen=True)
class VVAppConf:
    """
    A class to hold a vv app conf
    """

    # source directory
    directory: Path
    # compilation destination
    static: Path
    template: Path
    # compilation options
    is_partial: bool = field(default=False)


@dataclass
class VVConf:
    """A class to hold a vv conf"""

    # base settings paths
    base_dir: Path
    vv_base_dir: Path
    staticfiles_dir: Path
    templates_dir: Path
    static_url: str

    def to_json_str(self) -> str:
        """Generate an indented json string representation

        :return: indented json string
        :rtype: str
        """
        return json.dumps(asdict(self), indent=2, default=str)



from pathlib import Path
from typing import Dict

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from introspection.colors import colors

from vv.conf import VvConfManager
from vv.conf.models import VVAppConf
from vv.configure import (
    check_packages_dependencies,
    generate_packages_json_build_commands,
    generate_vite_compilation_config,
    packages_conf,
    write_conf,
)


class Command(BaseCommand):
    help = "Configure Vitejs to compile to Django static files and template"

    def add_arguments(self, parser):
        parser.add_argument(
            "--app",
            nargs="?",
            dest="frontend_app_dir",
            type=str,
            default=None,
            help="Generate a config for the specified frontend app folder",
        )
        parser.add_argument(
            "-w",
            action="store_true",
            dest="write",
            default=False,
            help="Write to the Vite config files, print the config if not set",
        )
        parser.add_argument(
            "-p",
            action="store_true",
            dest="is_partial",
            default=False,
            help="Compile to a partial template",
        )
        parser.add_argument(
            "--template",
            nargs="?",
            dest="template",
            type=str,
            default=None,
            help="The template to compile to",
        )
        parser.add_argument(
            "--static",
            type=str,
            nargs="?",
            dest="static",
            default=None,
            help="The static folder to compile to",
        )

    def handle(self, *args, **options):
        # verbosity = options["verbosity"]
        if settings.DEBUG is False:
            print("This command only works in debug mode: do not use in production")
            return
        # get the settings
        manager = VvConfManager()
        # print(manager.conf.vv_base_dir, options["frontend_app_dir"])
        app: VVAppConf
        if options["frontend_app_dir"] is not None:
            # print(f'Generating config for app {options["app"]}')
            app_path = manager.conf.vv_base_dir / options["frontend_app_dir"]
            if not app_path.exists():
                raise ValueError(
                    (
                        f'The folder {options["frontend_app_dir"]} does not exist in'
                        f"{manager.conf.vv_base_dir}"
                    )
                )
            static: str = app_path.name
            if options["static"] is not None:
                static = options["static"]
            template: str = "index.html"
            if options["template"] is not None:
                template = options["template"]
            else:
                if options["is_partial"] is True:
                    template = app_path.name + ".html"
            app_conf = manager.frontend_app_conf(
                app_path.name,
                template_path=template,
                static_path=static,
                is_partial=options["is_partial"],
            )
            app = app_conf
        else:
            print(
                "No frontend app dir given, searching for default a frontend folder..."
            )
            # check if a directory named "frontend" exists
            app_path = manager.conf.vv_base_dir / "frontend"
            if not app_path.exists():
                raise CommandError("No frontend directory found")
            template: str = "index.html"
            if options["template"] is not None:
                template = options["template"]
            else:
                if options["is_partial"] is True:
                    template = "frontend.html"
            app = manager.frontend_default_conf(
                template_path=template, is_partial=options["is_partial"]
            )
            static: str = app.directory.name
            if options["static"] is not None:
                static = options["static"]
        # print(f"App {app.directory.name} conf:", app)
        missing_template = False
        partial_template_front: Path
        if options["is_partial"] is True:
            # verify the existence of the partial template in the frontend app
            partial_template_relative = app.template.relative_to(
                manager.conf.templates_dir
            )
            partial_template_front = app.directory / partial_template_relative
            if partial_template_front.exists() is False:
                missing_template = True
        # generate conf
        viteconf = generate_vite_compilation_config(
            manager.conf, app, options["is_partial"]
        )
        packages_cmds: Dict[str, str] = generate_packages_json_build_commands(
            manager.conf, app
        )
        packages_file = app.directory / "package.json"
        dev_deps: Dict[str, str] = {}
        if options["write"] is True:
            write_conf(app.directory, viteconf)
            print("Writing package.json")
            _, dev_deps = packages_conf(packages_file, packages_cmds)
        else:
            print("-----------------------------------")
            print(f"Config for app {app.directory.name}")
            print("-----------------------------------")
            print(viteconf)
            print("---- packages.json build commands ----")
            for cmd_line in packages_cmds:
                print(f"{cmd_line} : {packages_cmds[cmd_line]}")
            _, dev_deps = packages_conf(packages_file, packages_cmds, read_only=True)
        # checking dev dependencies
        check_packages_dependencies(dev_deps)
        # print warnings
        if missing_template is True:
            warn_msg = (
                f"The {partial_template_front} file does not exist and "  # type: ignore
                "is required for compilation of partial template."
            )
            print(colors.yellow("warning"), warn_msg)
            resp = input(
                (
                    f"Create the {partial_template_front} "  # type: ignore
                    "template? [Y/n)"
                )
            )
            create = False
            if resp in ["", "Y", "y", "yes"]:
                create = True
            if create is False:
                msg = (
                    "You can create it with this content:\n"
                    '<div id="app"></div>\n'
                    '<script type="module" src="/src/main.ts"></script>'
                )
                print(msg)
                return
            else:
                with open(partial_template_front, "x") as f:  # type: ignore
                    content = (
                        '<div id="app"></div>\n'
                        '<script type="module" src="/src/main.ts"></script>'
                    )
                    f.write(content)
                    print(
                        colors.green("ok"),
                        f"{partial_template_front} written",  # type: ignore
                    )



from pathlib import Path
from typing import Tuple

from django.core.management.base import CommandError

from .model import FrontendModel


def check_for_file_overwrite(file: Path, default_yes: bool = True) -> Tuple[bool, bool]:
    """
    Check if a file exists and prompt the user for overwrite
    """
    exists = False
    abort = False
    if file.exists() is True:
        abort = True
        exists = True
        resp: str
        if default_yes is True:
            resp = input(f"The file {file} already exists. Overwrite? [Y/n)")
            if resp in ["", "Y", "y", "yes"]:
                abort = False
        else:
            resp = input(f"The file {file} already exists. Overwrite? [y/N)")
            if resp in ["Y", "y", "yes"]:
                abort = False
    return abort, exists


def write_tsmodel(model: FrontendModel, destination_folder: Path, verbose=True):
    """
    Write a frontend Typescript model to a folder
    """
    if not destination_folder.exists():
        raise FileNotFoundError(f"Folder {destination_folder} not found")
    if verbose is True:
        print(f"Writing model {model.model.name}")
    # write constructor params interface
    contract = model.interface()
    filename = destination_folder / "contract.ts"
    # check if file exists
    abort, exists = check_for_file_overwrite(filename)
    if abort is True:
        raise CommandError("Aborting")
    if verbose is True:
        print(f"Writing contract in {filename}")
    mode = "x"
    if exists is True:
        mode = "w"
    with open(filename, mode) as f:
        f.write(contract)
    # write the model class
    tsclass = model.tsclass()
    filename = destination_folder / "index.ts"
    # check if file exists
    abort, exists = check_for_file_overwrite(filename)
    if abort is True:
        raise CommandError("Aborting")
    if verbose is True:
        print(f"Writing class {model.model.name} in {filename}")
    mode = "x"
    if exists is True:
        mode = "w"
    with open(filename, mode) as f:
        f.write(tsclass)



import os
from vv.conf.models import VVAppConf
from vv.conf.manager import VvConfManager

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from introspection import AppInspector
from introspection.inspector import title, subtitle

from vv.frontend_models.model import FrontendModel
from vv.frontend_models.write import write_tsmodel


class Command(BaseCommand):
    help = "Create Typescript models for an app"

    def add_arguments(self, parser):
        parser.add_argument(
            "app",
            nargs="+",
            type=str,
            help="the Django app name to generate models from",
        )
        parser.add_argument(
            "--app",
            nargs="?",
            dest="frontend_app_dir",
            type=str,
            default=None,
            help="the frontend app to add models to. A models directory "
            "will be created into it if it does not exists",
        )
        parser.add_argument(
            "-w",
            action="store_true",
            dest="write",
            default=False,
            help="Write to the Vite config files, print the config if not set",
        )

    def handle(self, *args, **options):
        # verbosity = options["verbosity"]
        if settings.DEBUG is False:
            print("This command only works in debug mode: do not use in production")
            return
        if "app" not in options:
            raise CommandError("Provide an app to generate models for")
        # get the settings
        app_conf: VVAppConf
        manager = VvConfManager()
        if options["frontend_app_dir"] is None:
            try:
                app_conf = manager.frontend_default_conf()
            except FileNotFoundError as e:
                raise CommandError(e)
        else:
            app_conf = manager.frontend_app_conf(options["frontend_app_dir"])
        app_name = options["app"][0]
        app = AppInspector(app_name)
        app.get_models()
        for model in app.models:
            title(f"Model {model.name}")
            fm = FrontendModel(model)
            if options["write"] is False:
                print(fm.tsclass() + "\n")
                subtitle("Interface")
                print("\n" + fm.interface())
            else:
                models_dir = app_conf.directory / "src/models"
                if not models_dir.exists():
                    print("Creating models directory")
                    os.mkdir(models_dir)
                name = fm.snake_case_name
                dest_dir = models_dir / name
                if not dest_dir.exists():
                    print(f"Creating directory {name}")
                    os.mkdir(dest_dir)
                write_tsmodel(fm, dest_dir)



from django.core.management.base import BaseCommand

from introspection.colors import colors
from vv.conf import VvConfManager


class Command(BaseCommand):
    help = "Check your vv config and settings"

    def add_arguments(self, parser):
        parser.add_argument(
            "-i",
            action="store_true",
            dest="info",
            default=False,
            help="Display info about the settings found",
        )

    def handle(self, *args, **options):
        conf = VvConfManager()
        issues = conf.check_vv_settings(options["info"])
        if len(issues) > 0:
            print(f"Found {len(issues)} issues:")
            for issue in issues:
                print(colors.red("required"), issue)
        else:
            print(colors.green("ok"), "no issues found")



import os
import re
from pathlib import Path

from introspection.const import NUMBER_FIELDS, STRING_FIELDS


def rel_path(path: Path, relative_to: Path) -> Path:
    """
    Return a directory relative path to another directory
    """
    return Path(os.path.relpath(path, relative_to))


def to_snake_case(word: str) -> str:
    """
    Convert a string to snake case
    """
    return re.sub(r"(?<!^)(?=[A-Z])", "_", word).lower()


def to_camel_case_var(word: str) -> str:
    """
    Convert a string to camel case
    """
    s = "".join(x.capitalize() or "_" for x in word.split("_"))
    return s[0].lower() + s[1:]


def field_ts_type_from_classname(classname: str):
    """
    Get a field typescript type
    """
    if classname in NUMBER_FIELDS:
        return "number"
    elif classname in STRING_FIELDS:
        return "string"
    elif classname == "JSONField":
        return "Record<string, any>"
    elif classname == "BooleanField":
        return "boolean"
    raise KeyError(f"Type for field {classname} not found")



"""
Application URLs
"""
from django.urls import path
from django.contrib.auth.decorators import login_required

from .views.auth import login_view, logout_view


urlpatterns = [
    path("auth/login/", login_view),
    path("auth/logout/", login_required(logout_view)),
]



import json
from typing import Dict, List, Set, Tuple
from pathlib import Path
from vv.conf.models import VVAppConf, VVConf

from vv.utils import rel_path


def vite_conf_file(path: Path) -> Path:
    """
    Get a Vite config file from a path
    """
    jsconf = path / "vite.config.js"
    tsconf = path / "vite.config.ts"
    if jsconf.exists():
        print("Found a frontend/vite.config.js file")
        return jsconf
    if tsconf.exists():
        print("Found a frontend/vite.config.ts file")
        return tsconf
    raise ValueError(f"No Vite config file found for path {path}")


def rel_app_dirs(conf: VVConf, app: VVAppConf) -> Tuple[Path, Path, Path]:
    """
    Get an app's compilation output dirs
    """
    # print("Base dir:", conf.base_dir)
    rel_app_dir = rel_path(app.directory, conf.vv_base_dir)
    rel_static_dir = rel_path(app.static, app.directory)
    rel_template = rel_path(app.template, app.directory)
    # print("Base dir", conf.base_dir)
    # print("Static dir", app.static, rel_static_dir)
    return (rel_app_dir, rel_static_dir, rel_template)


def generate_vite_compilation_config(
    conf: VVConf, app: VVAppConf, is_partial: bool = False
) -> str:
    """
    Configure Vitejs compilation output dir
    """
    print(f"Configuring app {app.directory.name} with these compilation destinations:")
    rel_app_dir, rel_static_dir, rel_template = rel_app_dirs(conf, app)
    print("static assets -> ", rel_static_dir)
    print("template -> ", rel_template)
    # buf: List[str] = []
    buf: List[str] = [f'\tpublicDir: "{rel_static_dir}",']
    buf.append("\tbuild: {")
    buf.append("\t\temptyOutDir: false,")
    buf.append("\t\trollupOptions: {")
    buf.append("\t\t\toutput: {")
    buf.append(f'\t\t\t\tdir: "{rel_static_dir}"')
    # app_name = app_dir.name
    # buf.append(f'\t\t\tchunkFileNames: "{app_name}/[name]-[hash].js"')
    # buf.append(f'\t\t\tentryFileNames: "{app_name}/[name].js"')
    # buf.append(f'\t\t\tassetFileNames: "{app_name}/[name]-[hash][extname]"')
    if is_partial is True:
        buf.append("\t\t\t},")
        buf.append("\t\t\tinput: {")
        buf.append(f'\t\t\t\tapp: "{app.template.relative_to(conf.templates_dir)}"')
        buf.append("\t\t\t}")
    else:
        buf.append("\t\t\t}")
    buf.append("\t\t}")
    buf.append("\t}")
    return "\n".join(buf)


def write_conf(app_dir: Path, viteconf: str, verbose: bool = True) -> None:
    """
    Write the Vite conf for an app
    """
    filename = vite_conf_file(app_dir)
    if verbose is True:
        print(f"Writing config in {filename.name}")
    lines = open(filename, "r").readlines()
    new_pre_last_line = lines[-2].rstrip()
    if not new_pre_last_line.endswith(","):
        new_pre_last_line = new_pre_last_line + ","
    new_pre_last_line = new_pre_last_line + "\n"
    new_last_line = lines[-1].rstrip().replace("})", viteconf + "\n})")
    lines[-2] = new_pre_last_line
    lines[-1] = new_last_line
    open(filename, "w").writelines(lines)


def generate_packages_json_build_commands(
    conf: VVConf, app: VVAppConf
) -> Dict[str, str]:
    """
    Generate the package.json commands for the build
    """
    rel_app_dir, rel_static_dir, rel_template = rel_app_dirs(conf, app)
    buf: Dict[str, str] = {
        "build:prepare": f"del {rel_static_dir}/assets/* {rel_template} --force"
    }
    buf["build:build"] = f"vite build --base={conf.static_url}{rel_app_dir.name}/"
    buf[
        "build:moveindex"
    ] = f"move-file {rel_static_dir}/{rel_template.name} {rel_template}"
    buf["build:clean"] = "del ./dist"
    buf["build"] = "run-s build::prepare build::build build::moveindex build:clean"
    return buf


def packages_conf(
    packages_conf_path: Path, cmds: Dict[str, str], read_only: bool = False
) -> Tuple[Dict[str, str], Dict[str, str]]:
    """
    Write the packages.json config file and return the list
    of packages dependencies and dev dependencies found
    """
    deps: Dict[str, str] = {}
    dev_deps: Dict[str, str] = {}
    with open(packages_conf_path, "r+") as jsonFile:
        data = json.load(jsonFile)
        # print(data["scripts"])
        # print("CMDS", cmds)
        deps = data["dependencies"]
        dev_deps = data["devDependencies"]
        for cmd in cmds:
            data["scripts"][cmd] = cmds[cmd]
        if not read_only:
            jsonFile.seek(0)
            json.dump(data, jsonFile, indent=4)
            jsonFile.truncate()
    return deps, dev_deps


def check_packages_dependencies(packages: Dict[str, str]):
    """
    Check if the npm dependencies required for compilation
    are installed in package.json
    """
    required: Set[str] = {"del-cli", "npm-run-all", "move-file-cli"}
    if not required.issubset(packages):
        print("Some npm dev dependencies are missing and require to be installed:")
        missing: Set[str] = set()
        for req in required:
            if not {req}.issubset(packages):
                missing.add(req)
        line = " ".join(missing)
        print(f"yarn add -D {line}")
        print("# or")
        print(f"npm install {line} --save-dev")



import os
from pathlib import Path
from shutil import copy

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from introspection.inspector.inspector import AppInspector

from vv import files
from vv.conf.manager import VvConfManager
from vv.conf.models import VVAppConf
from vv.frontend_models.model import FrontendModel


class Command(BaseCommand):
    help = "Create a Typescript api for existing models"

    def add_arguments(self, parser):
        parser.add_argument(
            "app",
            nargs="+",
            type=str,
            help="the Django app name to generate api models for",
        )
        parser.add_argument(
            "--app",
            nargs="?",
            dest="frontend_app_dir",
            type=str,
            default=None,
            help="the frontend app models to add api to. An api directory "
            "will be created",
        )

    def handle(self, *args, **options):
        # verbosity = options["verbosity"]
        if settings.DEBUG is False:
            print("This command only works in debug mode: do not use in production")
            return
        # get the settings
        app_conf: VVAppConf
        manager = VvConfManager()
        if options["frontend_app_dir"] is None:
            try:
                app_conf = manager.frontend_default_conf()
            except FileNotFoundError as e:
                raise CommandError(e)
        else:
            app_conf = manager.frontend_app_conf(options["frontend_app_dir"])
        files_path = Path(os.path.dirname(files.__file__))
        api_path = app_conf.directory / "src/api"
        rel_src_path = (app_conf.directory / "src").relative_to(
            manager.conf.vv_base_dir
        )
        if api_path.exists() is True:
            print(
                (
                    f"The api directory already exists in {rel_src_path}, skipping api "
                    "files copy"
                )
            )
        else:
            print(
                f"Creating directory {api_path.relative_to(manager.conf.vv_base_dir)}"
            )
            os.mkdir(api_path)
            print(f"Adding api files in {rel_src_path} ...")
            # copy(files_path / "api/model.ts", api_path)
            # copy(files_path / "api/interface.ts", api_path)
            copy(files_path / "api/index.ts", api_path)
            copy(files_path / "api/api.ts", api_path)
        # check dirs
        models_dir = app_conf.directory / "src/models"
        if models_dir.exists() is False:
            print(f"No models directory {models_dir} found, exiting")
            return
        app_name = options["app"][0]
        app = AppInspector(app_name)
        app.get_models()
        for mod in app.models:
            model = FrontendModel(mod)
            folder_path = models_dir / f"{model.snake_case_name}"
            print(f"Updating model in {folder_path}")
            filename = f"{folder_path}/index.ts"
            # print(f"Writing config in {filename.name}")
            lines = open(filename, "r").readlines()
            lines[0] = model.api_relative_import + ";\n" + lines[0]
            new_last_line = (
                lines[-1].rstrip().replace("}", f"\n{model.load_method()}\n" + "}")
            )
            lines[-1] = new_last_line
            open(filename, "w").writelines(lines)
        print("Install the dependencies in the frontend:")
        print("yarn add js-cookie @snowind/api")
        print("or npm install js-cookie @snowind/api")



# pyright: reportUnboundVariable=false
from pathlib import Path
from typing import Dict, Set, Tuple, Union

from django.conf import settings

from vv.conf.models import VVAppConf, VVConf


class VvConfManager:
    """
    A class to manage vv config
    """

    conf: VVConf

    def __init__(self) -> None:
        self.conf = self.from_settings()

    def from_settings(self) -> VVConf:
        """Get a vv app conf from settings

        :return: a vv apps conf
        :rtype: VVConf
        """
        (
            BASE_DIR,
            VV_BASE_DIR,
            STATICFILES_DIR,
            TEMPLATES_DIR,
            STATIC_URL,
        ) = self.read_settings()
        self.conf = VVConf(
            base_dir=BASE_DIR,
            vv_base_dir=VV_BASE_DIR,
            staticfiles_dir=STATICFILES_DIR,
            templates_dir=TEMPLATES_DIR,
            static_url=STATIC_URL,
        )
        return self.conf

    def frontend_default_conf(
        self,
        template_path: str = "",
        static_path: str = "",
        is_partial: bool = False,
    ):
        path = Path(self.conf.vv_base_dir / "frontend")
        if not path.exists():
            msg = (
                "No frontend folder found, you might create one in "
                f"{self.conf.base_dir} with a Vite command like:"
                "yarn create vite frontend --template=vue-ts"
            )
            raise FileNotFoundError(msg)
        return self.frontend_app_conf(
            "frontend", template_path, static_path, is_partial
        )

    def frontend_app_conf(
        self,
        dir_path: str,
        template_path: str = "",
        static_path: str = "",
        is_partial: bool = False,
    ) -> VVAppConf:
        """Get a frontend app conf from a path. If the app exists
        in VITE_APPS return it's conf or generate a default conf
        for this path

        :param path: the path to get a conf for
        :type path: Path
        :raises FileNotFoundError: if the path does not exist
        :return: a vv app conf
        :rtype: VVAppConf
        """
        app_dir = self.conf.vv_base_dir / dir_path
        if app_dir.exists() is False:
            raise FileNotFoundError(f"frontend app path {app_dir} not found")
        app_conf: Dict[str, Path] = {
            "directory": app_dir,
            "static": self.conf.staticfiles_dir / app_dir.name,
            "template": self.conf.templates_dir / "index.html",
        }
        if template_path != "":
            app_conf["template"] = self.conf.templates_dir / template_path
        if static_path != "":
            app_conf["static"] = self.conf.staticfiles_dir / static_path
        return VVAppConf(**app_conf, is_partial=is_partial)

    def read_app_conf(self, app_conf: Dict[str, Union[Path, bool]]) -> VVAppConf:
        """Read an app conf from a VITE_APPS setting

        :param app_conf: [description]
        :type app_conf: VVAppConf
        :return: [description]
        :rtype: VVAppConf
        """
        is_partial: bool = False
        params: Dict[str, Path] = {}
        for param in app_conf:
            if param == "is_partial":
                is_partial = bool(app_conf[param])
            else:
                params[param] = app_conf[param]  # type: ignore
        return VVAppConf(**params, is_partial=is_partial)

    def read_settings(self) -> Tuple[Path, Path, Path, Path, str]:
        """Check if the required settings are there and get them
        as Path

        :raises ValueError: if some required setting is not found
        :return: a tuple with base_dir, staticfiles_dir, templates_dir paths and
        a static url string
        :rtype: Tuple[Path, Path, Path, str]
        """
        base_dir: Path
        vv_base_dir: Path
        statifiles_dir: Path
        templates_dir: Path
        # make sure all the required settings are there
        if not hasattr(settings, "BASE_DIR"):
            raise ValueError("Missing BASE_DIR setting")
        # static
        if not hasattr(settings, "STATICFILES_DIRS"):
            raise ValueError("Missing STATICFILES_DIRS setting")
        if not hasattr(settings, "STATICFILES_DIRS"):
            raise ValueError("Missing STATIC_URL setting")
        else:
            if len(settings.STATICFILES_DIRS) == 0:
                raise ValueError("STATICFILES_DIRS setting is empty")
        # templates
        if not hasattr(settings, "TEMPLATES"):
            raise ValueError("Missing TEMPLATES setting")
        else:
            if len(settings.TEMPLATES) == 0:
                raise ValueError("Empty TEMPLATES setting")
            if "DIRS" not in settings.TEMPLATES[0]:
                raise ValueError('Missing TEMPLATES["DIRS"] setting')
            if len(settings.TEMPLATES[0]["DIRS"]) == 0:
                raise ValueError('TEMPLATES["DIRS"] setting is empty')
        # get the settings and make sure they are paths
        if isinstance(settings.BASE_DIR, str):
            base_dir = Path(settings.BASE_DIR)
        else:
            base_dir = settings.BASE_DIR
        if isinstance(settings.STATICFILES_DIRS[0], str):
            statifiles_dir = Path(settings.STATICFILES_DIRS[0])
        else:
            statifiles_dir = settings.STATICFILES_DIRS[0]
        if isinstance(settings.TEMPLATES[0]["DIRS"][0], str):
            templates_dir = Path(settings.TEMPLATES[0]["DIRS"][0])
        else:
            templates_dir = settings.TEMPLATES[0]["DIRS"][0]
        # check for a VV_BASE_DIR or set to BASE_DIR parent
        vv_base_dir: Path = getattr(settings, "VV_BASE_DIR", base_dir.parent)
        if hasattr(settings, "STATIC_URL") is False:
            raise ValueError("No STATIC_URL found")
        return (
            base_dir,
            vv_base_dir,
            statifiles_dir,
            templates_dir,
            str(settings.STATIC_URL),
        )

    def check_vv_settings(self, verbose: bool = False) -> Set[str]:
        """Read settings and check for issues

        :param verbose: print info, defaults to False
        :type verbose: bool, optional
        :return: a set of issues, empty if no issues is found
        :rtype: Set[str]
        """
        issues: Set[str] = set()
        if verbose is True:
            print("Checking base settings...")
        try:
            (
                BASE_DIR,
                VV_BASE_DIR,
                STATICFILES_DIR,
                TEMPLATES_DIR,
                STATIC_URL,
            ) = self.read_settings()
        except ValueError as e:
            issues.add(f"Missing setting: {e}")
        if verbose is True:
            print(f"BASE_DIR: {BASE_DIR}")
            print(f"VV_BASE_DIR: {VV_BASE_DIR}")
            print(f"STATICFILES_DIR: {STATICFILES_DIR}")
            print(f"TEMPLATES_DIR: {TEMPLATES_DIR}")
            print(f"STATIC_URL: {STATIC_URL}")
        if verbose is True:
            print("Checking the VITE_APPS setting...")
        if hasattr(settings, "VITE_APPS") is True:
            if verbose is True:
                print("Found a VITE_APPS setting:")
                print(self.from_settings().to_json_str())
        else:
            if verbose is True:
                print("No VITE_APPS setting found")
        return issues
